@@ -1,4 +1,5 @@
 """Parse datasets TLV encoded as specified by Thread."""
+
 from __future__ import annotations
 
 from dataclasses import dataclass, field
@@ -50,6 +51,8 @@ class MeshcopTLVType(IntEnum):
     PERIOD = 55
     SCAN_DURATION = 56
     ENERGY_LIST = 57
+    # Seen in a dataset imported through iOS companion app
+    APPLE_TAG_UNKNOWN = 74
     DISCOVERYREQUEST = 128
     DISCOVERYRESPONSE = 129
     JOINERADVERTISEMENT = 241
